unit mainform;

{$mode objfpc}{$H+}

interface

(*
  https://wiki.freepascal.org/TIpHtmlPanel
  https://forum.lazarus.freepascal.org/index.php/topic,56227.msg417837.html#msg417837
*)

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Menus, ExtCtrls,
  StdCtrls, SynEdit, SynHighlighterHTML, IpHtml, IpFileBroker;

type

  { TForm1 }

  TForm1 = class(TForm)
    BTOpen: TButton;
    BTFind: TButton;
    BTQuit: TButton;
    IpHtmlDataProvider1: TIpHtmlDataProvider;
    IpHtmlPanel1: TIpHtmlPanel;
    LBWait: TLabel;
    MainMenu1: TMainMenu;
    MIDemo: TMenuItem;
    MIQuit: TMenuItem;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    Splitter1: TSplitter;
    SynEdit1: TSynEdit;
    SynHTMLSyn1: TSynHTMLSyn;
    procedure BTFindClick(Sender: TObject);
    procedure BTOpenClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure IpHtmlDataProvider1GetImage(Sender: TIpHtmlNode; const URL: string; var Picture: TPicture);
    procedure MIQuitClick(Sender: TObject);
    procedure SynEdit1Change(Sender: TObject);
  private
    procedure Preview;
    procedure LoadRtf(const ARtfFileName: TFileName);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  DefaultTranslator;

var
  FPandocPath: TFileName;

(* ========================================================================== *)

type
  TPandocExecutionThread = class(TThread)
  private
    FDocxFileName: TFileName;
    FHtmlFileName: TFileName;
    procedure UpdateForm;
  protected
    procedure Execute; override;
  public
    constructor Create(const ARtfFileName: TFileName);
  end;

constructor TPandocExecutionThread.Create(const ARtfFileName: TFileName);
const
  CCreateSuspended = TRUE;
begin
  inherited Create(CCreateSuspended);
  FreeOnTerminate := TRUE;
  FDocxFileName := ARtfFileName;
end;

procedure TPandocExecutionThread.UpdateForm;
begin
  Form1.SynEdit1.Lines.LoadFromFile(FHtmlFileName);
  Form1.SynEdit1Change(nil);
  Form1.LBWait.Caption := '';
end;

procedure TPandocExecutionThread.Execute;
begin
  FHtmlFileName := ChangeFileExt(FDocxFileName, '.html');
  ExecuteProcess(FPandocPath, ['-s', FDocxFileName, '-t', 'html', '-o', FHtmlFileName, '--extract-media=.']);
  Synchronize(@UpdateForm);
end;

(* ========================================================================== *)

procedure TForm1.SynEdit1Change(Sender: TObject);
begin
  Preview;
end;

procedure TForm1.FormCreate(Sender: TObject);
const
  CDefaultContent = '<html><body>Bitte wählen Sie eine RTF-Datei aus.</body></html>';
begin
{$IFDEF MSWINDOWS}
  FPandocPath := 'pandoc.exe';
{$ELSE}
  FPandocPath := '/usr/bin/pandoc';
{$ENDIF}
  BTOpen.Enabled := FileExists(FPandocPath);

  SynEdit1.Lines.Text := CDefaultContent;
  //SynEdit1.Lines.LoadFromFile('LIESMICH.html');
  SynEdit1Change(nil);

  LBWait.Caption := '';
end;

procedure TForm1.BTOpenClick(Sender: TObject);
begin
  OpenDialog1.Filter := 'Rich Text Format|*.rtf';
  OpenDialog1.InitialDir := GetCurrentDir;
  if OpenDialog1.Execute then
  begin
    LoadRtf(OpenDialog1.FileName);
  end;
end;

procedure TForm1.BTFindClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
  OpenDialog1.Filter := 'Pandoc Executable|pandoc.exe';
{$ELSE}
  OpenDialog1.Filter := 'Pandoc Executable|pandoc';
{$ENDIF}
  OpenDialog1.InitialDir := GetCurrentDir;
  if OpenDialog1.Execute then
  begin
    FPandocPath := OpenDialog1.FileName;
    BTOpen.Enabled := FileExists(FPandocPath);
  end;
end;

procedure TForm1.FormActivate(Sender: TObject);
(* www.developpez.net/forums/showthread.php?t=2089611#post11610251 *)
var
  LMenuBarHeight: integer;
begin
  LMenuBarHeight := Height - ClientHeight;
  Height := Panel1.Height + LMenuBarHeight;
  Panel1.Anchors := Panel1.Anchors + [akBottom, akRight];
end;

procedure TForm1.IpHtmlDataProvider1GetImage(Sender: TIpHtmlNode; const URL: string; var Picture: TPicture);
begin
  if FileExists(URL) then
  begin
    Picture := TPicture.Create;
    Picture.LoadFromFile(URL);
  end;
end;

procedure TForm1.MIQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Preview;
var
  LStream: TMemoryStream;
begin
  LStream := TMemorystream.Create;
  try
    SynEdit1.Lines.SaveToStream(LStream);
    LStream.Position := 0;
    IpHtmlPanel1.SetHtmlFromStream(LStream);
  finally
    LStream.free;
  end;
end;

procedure TForm1.LoadRtf(const ARtfFileName: TFileName);
var
  LThread: TPandocExecutionThread;
begin
  LBWait.Caption := 'Datei-Konvertierung...';
  LThread := TPandocExecutionThread.Create(ARtfFileName);
  LThread.Start;
end;

end.

